<?php
/*

Copyright (c)2002-2003 DuckCorp(tm) and RtpNet(tm)



This file is part of LMS.

LMS is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

LMS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with LMS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

class LdapConnection
{
	var	$lc_connected, $lc_server, $lc_ds, $lc_error, $lc_display_errors;
	var	$lc_binded, $lc_bind_dn, $lc_bind_passwd;
	var	$lc_has_wrapping_parameters, $lc_base_dn, $lc_user_part_dn, $lc_commonuser_part_dn, $lc_adminuser_part_dn, $lc_serviceuser_part_dn;
	var	$lc_ref;

	function LdapConnection()
	{
		$this->_init();
		register_shutdown_function(array(&$this, "LdapConnection_destructor"));
	}

	function LdapConnection_destructor()
	{
		if ($this->$this->lc_connected)
			$this->disconnect();
	}

	function _init()
	{
		$this->lc_server = "";
		$this->lc_connected = false;
		$this->lc_ds = 0;
		$this->lc_error = "";
		$this->lc_display_errors = false;
		$this->lc_binded = false;
		$this->lc_bind_dn = "";
		$this->lc_bind_passwd = "";
		$this->lc_has_wrapping_parameters = "";
		$this->lc_base_dn = "";
		$this->lc_user_part_dn = "";
		$this->lc_commonuser_part_dn = "";
		$this->lc_adminuser_part_dn = "";
		$this->lc_serviceuser_part_dn = "";
	}

	function connect($server, $checkv3)
	{
		$this->lc_server = $server;
		
		if ($this->lc_connected)
			$this->disconnect();

		$this->lc_ds = ldap_connect($this->lc_server);
		if ($this->lc_ds)
		{
			if ($checkv3 && !ldap_set_option($this->lc_ds, LDAP_OPT_PROTOCOL_VERSION, 3))
			{
				$this->disconnect();
				$this->_set_error("Failed to set protocol version to 3");
			}
			else
			{
				$this->lc_connected = true;
				$this->_set_error();
				return true;
			}
		}
		else
			$this->_set_error("Cannot find LDAP Server");

		return false;
	}

	function disconnect()
	{
		if (!$this->lc_connected)
		{
			$this->_set_error("You are not connected to any server");
			return false;
		}

		ldap_close($this->lc_ds);
		$this->_init();
		return true;
	}

	function bind($bind_dn, $bind_passwd)
	{
		if (!$this->lc_connected)
		{
			$this->_set_error("You are not connected to any server");
			return false;
		}

		if (ldap_bind($this->lc_ds, $bind_dn, $bind_passwd))
		{
			$this->lc_binded =true;
			$this->lc_bind_dn = $bind_dn;
			$this->lc_bind_passwd = $bind_passwd;
			
			$this->_set_error();
			return true;
		}

		$this->_set_error("Cannot Bind as '".$bind_dn."'");
		return false;
	}

	function last_error()
	{
		return $this->lc_error;
	}

	function set_wrapping_parameter($base_dn, $user_part_dn, $commonuser_part_dn, $adminuser_part_dn, $serviceuser_part_dn)
	{
		$this->lc_has_wrapping_parameters = true;
		$this->lc_base_dn = $base_dn;
		$this->lc_user_part_dn = $user_part_dn;
		$this->lc_commonuser_part_dn = $commonuser_part_dn;
		$this->lc_adminuser_part_dn = $adminuser_part_dn;
		$this->lc_serviceuser_part_dn = $serviceuser_part_dn;

		$this->_set_error();
		return true;
	}
	
	function w_bind($bind_user, $bind_passwd)
	{
		if (!$this->lc_connected)
		{
			$this->_set_error("Not connected to any server");
			return false;
		}
		if (!$this->lc_has_wrapping_parameters)
		{
			$this->_set_error("Wrapping parameters not set");
			return false;
		}

		$bind_dn = "uid=".$bind_user.",".$this->lc_commonuser_part_dn.",".$this->user_part_dn.",".$this->lc_base_dn;
		if (!ldap_bind($this->lc_ds, $bind_dn, $bind_passwd))
		{
			$bind_dn = "uid=".$bind_user.",".$this->lc_adminuser_part_dn.",".$this->user_part_dn.",".$this->lc_base_dn;
			if (!ldap_bind($this->lc_ds, $bind_dn, $bind_passwd))
			{
				$bind_dn = "uid=".$bind_user.",".$this->lc_serviceuser_part_dn.",".$this->user_part_dn.",".$this->lc_base_dn;
				if (!ldap_bind($this->lc_ds, $bind_dn, $bind_passwd))
				{
					$this->_set_error("Cannot Bind as '".$bind_user."'");
					return false;
				}
			}
		}
		
		$this->lc_binded =true;
		$this->lc_bind_dn = $bind_dn;
		$this->lc_bind_passwd = $bind_passwd;
			
		$this->_set_error();
		return true;
	}

	function set_display_errors($display_errors)
	{
		$this->lc_display_errors = $display_errors;
	}

	function _set_error($err = "")
	{
		$this->lc_error = $err;

		if ($this->lc_display_errors)
			if ($this->lc_error)
			print("### ".$this->lc_error." ###\n");
	}

	function &search($base, $search)
	{
		if (!$this->lc_connected)
		{
			$this->_set_error("Not connected to any server");
			return false;
		}
		if (!$this->lc_binded)
		{
			$this->_set_error("Not binded to any account");
			return false;
		}

		$sr = ldap_search($this->lc_ds, $base, $search);
		if ($sr)
		{
			$nb = ldap_count_entries($this->lc_ds, $sr);
			if (!$nb)
				$tab = array();
			else
				$tab = ldap_get_entries($this->lc_ds, $sr);

			$this->lc_ref = new LdapEntries($sr, $tab, $nb);
			$this->lc_ref->get_count();
	
			return $this->lc_ref;
		}

		$this->_set_error("LDAP Error : ".ldap_error($this->lc_ds));
		return null;
	}

	function &w_search($search)
	{
		if (!$this->_prerequisite(true, true))
			return null;

		return $this->search($this->lc_base_dn, $search);
	}

	function _prerequisite($need_bind, $need_wrapping_parameters)
	{
		if (!$this->lc_connected)
		{
			$this->_set_error("Not connected to any server");
			return false;
		}
		if ($need_bind && !$this->lc_binded)
		{
			$this->_set_error("Not binded to any account");
			return false;
		}
		if ($need_wrapping_parameters && !$this->lc_has_wrapping_parameters)
		{
			$this->_set_error("Wrapping parameters not set");
			return false;
		}

		return true;
	}
}

class LdapEntries
{
	var	$le_sr, $le_tab, $le_nb;

	function LdapEntries($sr, $tab, $nb)
	{
		$this->le_sr = $sr;
		$this->le_tab = $tab;
		$this->le_nb = $nb;

		register_shutdown_function(array(&$this, "LdapEntries_destructor"));
	}

	function LdapEntries_destructor()
	{
		ldap_free_result($this->le_sr);
	}
	
	function get_count()
	{
		return $this->le_nb;
	}

	function get_entry($i)
	{
		return $this->le_tab[$i];
	}

	function set_entry($entry, $i)
	{
		$this->le_tab[$i] = $entry;
	}

	function sort_entry_by_attr($attr)
	{
	}
}

?>
